package new_market

import (
	"context"
	"fmt"
	"github.com/mitchellh/mapstructure"
	"github.com/sleepinggodoflove/lansexiongdi-marketing-sdk/api/v1/key"
	"github.com/sleepinggodoflove/lansexiongdi-marketing-sdk/core"
	"trasfer_middleware/cmd/rpc/etc"
	mqs "trasfer_middleware/until/mq"

	"trasfer_middleware/cmd/rpc/internal/logic/po"
	"trasfer_middleware/cmd/rpc/internal/logic/po/new_market/types"
	"trasfer_middleware/cmd/rpc/internal/logic/vo"
	"trasfer_middleware/cmd/rpc/pb/transfer"
	"trasfer_middleware/until/common"
	"trasfer_middleware/until/sysLog"
)

type NewMarketStruct struct {
	Conf       *types.NewMarketConf
	AppId      string
	CoreConfig *core.Config
	*po.RequestStruct
	rmq *mqs.Producer
}

func NewNewMarket(conf types.NewMarketConf, rmq *mqs.Producer) *NewMarketStruct {

	return &NewMarketStruct{
		Conf: &conf,
		CoreConfig: &core.Config{
			PrivateKey: conf.PRK,
			PublicKey:  conf.PUK,
			Key:        conf.KEY,
			BaseURL:    conf.Host,
		},
		rmq:           rmq,
		RequestStruct: &po.RequestStruct{},
	}
}

func (r *NewMarketStruct) Init(config etc.RockerMqConfig) *NewMarketStruct {
	r.RequestStruct.Config = config
	return r
}

func (r *NewMarketStruct) handleLog(ctx context.Context, url string, resp *key.Response, in interface{}) error {
	r.SetRequestStruct(in)
	err := r.sendMq(ctx, url, resp)
	return err
}

func (r *NewMarketStruct) Order(ctx context.Context, request *transfer.NewMarketOrderReq) (res *transfer.NewMarketOrderRes, err error) {

	coreKey, err := r.coreKey(request.AppId)
	if err != nil {
		return nil, err
	}
	response, err := coreKey.Order(ctx, &key.OrderRequest{
		OutBizNo:   request.OutBizNo,
		ActivityNo: request.ActivityNo,
		Number:     request.Number,
	})

	if err != nil {
		return nil, err
	}

	_ = mapstructure.Decode(response, &res)

	r.handleLog(ctx, vo.NEW_MARKET_ORDER, response, request)
	return res, err
}

func (r *NewMarketStruct) Query(ctx context.Context, request *transfer.NewMarketQueryReq) (res *transfer.NewMarketOrderRes, err error) {
	coreKey, err := r.coreKey(request.AppId)
	if err != nil {
		return nil, err
	}
	response, err := coreKey.Query(ctx, &key.QueryRequest{
		OutBizNo: request.OutBizNo,
		TradeNo:  request.TradeNo,
	})
	if err != nil {
		return nil, err
	}
	_ = mapstructure.Decode(response, &res)
	r.handleLog(ctx, vo.NEW_MARKET_ORDER, response, request)
	return res, err
}

func (r *NewMarketStruct) Discard(ctx context.Context, request *transfer.NewMarketDiscardReq) (res *transfer.NewMarketDiscardRes, err error) {
	coreKey, err := r.coreKey(request.AppId)
	if err != nil {
		return nil, err
	}
	response, err := coreKey.Discard(ctx, &key.DiscardRequest{
		OutBizNo: request.OutBizNo,
		TradeNo:  request.TradeNo,
	})
	if err != nil {
		return nil, err
	}
	_ = mapstructure.Decode(response, &res)
	r.handleLog(ctx, vo.NEW_MARKET_DISCARD, response, request)
	return res, err
}

func (r *NewMarketStruct) coreKey(appId string) (*key.Key, error) {
	r.CoreConfig.AppID = appId
	coreData, err := core.NewCore(r.CoreConfig)

	if err != nil {
		return nil, err
	}

	return &key.Key{Core: coreData}, nil
}

func (r *NewMarketStruct) sendMq(ctx context.Context, url string, resp *key.Response) error {
	//异步存入请求记录
	topic := r.RequestStruct.Config.TopicPrefix + r.RequestStruct.Config.Topic.NewMarket.Name
	err := r.rmq.SendSync(ctx, topic, po.SetMqSendDataNewMarket(r.RequestStruct, resp, url))
	if err != nil {
		sysLog.LogSendMq(ctx, err)
	}
	return err
}

func (r *NewMarketStruct) SetRequestStruct(in interface{}) {
	data := common.StructToMap(in)
	requestBody := make(map[string]string, len(data))
	for key, value := range data {
		requestBody[key] = fmt.Sprintf("%v", value)
	}
	r.RequestStruct.RequestBody = requestBody
}
